# -*- coding: utf-8 -*-
from pagador import settings

from pagador.reloaded import entidades
from pagador_rede.reloaded import cadastro

CODIGO_GATEWAY = 11
STATUS_REVISAO = '1127'


class Authentication(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'acquirer_code': 'AcquirerCode'}
    _atributos = ['password']


class AcquirerCode(entidades.BaseParaPropriedade):
    _atributos = ['rdcd_pv']


class Transaction(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'txn_details': 'TxnDetails', 'card_txn': 'CardTxn', 'historic_txn': 'HistoricTxn'}

    def __init__(self, **dados):
        super(Transaction, self).__init__(**dados)
        self._nao_serializar = []


class CardTxn(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'card': 'Card'}
    _atributos = ['method']


class HistoricTxn(entidades.BaseParaPropriedade):
    _atributos = ['reference', 'method', 'authcode']


class Card(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'cv2_avs': 'Cv2Avs'}
    _atributos = ['pan', 'expirydate', 'card_account_type']


class Cv2Avs(entidades.BaseParaPropriedade):
    _atributos = ['cv2']


class TxnDetails(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'instalments': 'Instalments', 'risk': 'Risk'}
    _parametros_para_serializacao = {'amount': {'currency': 'BRL'}}
    _atributos = ['dba', 'merchantreference', 'capturemethod', 'amount']


class Instalments(entidades.BaseParaPropriedade):
    _atributos = ['type', 'number']


class Risk(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'action': 'Action'}


class Action(entidades.BaseParaPropriedade):
    _parametros_para_serializacao = {'self': {'service': '1'}}
    _chaves_alternativas_para_serializacao = {'merchant_configuration': 'MerchantConfiguration', 'customer_details': 'CustomerDetails'}


class MerchantConfiguration(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'callback_configuration': 'CallbackConfiguration'}
    _atributos = ['merchant_location', 'channel']


class CallbackConfiguration(entidades.BaseParaPropriedade):
    _atributos = ['callback_format', 'callback_url', 'callback_options']


class CustomerDetails(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {
        'shipping_details': 'ShippingDetails',
        'risk_details': 'RiskDetails',
        'personal_details': 'PersonalDetails',
        'address_details': 'AddressDetails',
        'payment_details': 'PaymentDetails',
        'order_details': 'OrderDetails'
    }


class ShippingDetails(entidades.BaseParaPropriedade):
    _atributos = ['title', 'first_name', 'surname', 'address_line1', 'address_line2', 'city', 'state_province', 'country', 'zip_code', 'delivery_date', 'delivery_method']


class RiskDetails(entidades.BaseParaPropriedade):
    _atributos = ['account_number', 'email_address', 'session_id', 'ip_address', 'user_id', 'usermachine_id', 'user_profile']


class PersonalDetails(entidades.BaseParaPropriedade):
    _atributos = ['first_name', 'surname', 'telephone', 'telephone_2', 'date_of_birth', 'nationality', 'id_number', 'id_type', 'ssn']


class AddressDetails(entidades.BaseParaPropriedade):
    _atributos = ['address_line1', 'address_line2', 'city', 'state_province', 'country', 'zip_code']


class PaymentDetails(entidades.BaseParaPropriedade):
    _atributos = ['payment_method']


class OrderDetails(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'billing_details': 'BillingDetails', 'line_items': 'LineItems'}
    _atributos = ['discount_value', 'time_zone', 'proposition_date']


class BillingDetails(entidades.BaseParaPropriedade):
    _atributos = ['name', 'address_line1', 'address_line2', 'city', 'state_province', 'country', 'zip_code']


class LineItems(entidades.BaseParaPropriedade):
    _chaves_alternativas_para_serializacao = {'item': 'Item'}


class Item(entidades.BaseParaPropriedade):
    _atributos = ['product_code', 'product_description', 'product_category', 'order_quantity', 'unit_price']


class PassosDeEnvio(object):
    auth = 'auth'
    pre = 'pre'
    accept_review = 'accept_review'
    fulfill = 'fulfill'


class TipoDeCartao(object):
    credito = 'credit'
    debito = 'debit'


class TipoDeParcelamento(object):
    com_juros = 'interest_bearing'
    sem_juros = 'zero_interest'


class MetodoDeCaptura(object):
    ecomm = "ecomm"
    cont_auth = "cont_auth"


class ChannelValues(object):
    web = "W"


class MeiosDePagamento(object):
    credito = "CC"
    debito = "DB"


class CallbackFormat(object):
    http = "HTTP"
    xml = "XML"
    soap = "SOAP"


class TipoEndereco(object):
    cliente = 1
    entrega = 2
    pagamento = 3


class Malote(entidades.Malote):
    _parametros_para_serializacao = {'self': {'version': '2'}}
    _chaves_alternativas_para_serializacao = {'self': 'Request', 'authentication': 'Authentication', 'transaction': 'Transaction'}

    def __init__(self, configuracao):
        super(Malote, self).__init__(configuracao)
        self.authentication = None
        self.transaction = None
        self._passo_atual = PassosDeEnvio.pre
        self._pedido = None
        self._dados = {}

    def monta_conteudo(self, pedido, parametros_contrato=None, dados=None):
        transaction = None
        self._dados = dados
        self._passo_atual = dados['passo']
        self._pedido = pedido
        pedido_pagamento = entidades.PedidoPagamento(self.configuracao.loja_id, pedido.numero, self.configuracao.meio_pagamento.codigo)
        pedido_pagamento.preencher_do_banco()
        if pedido_pagamento.identificador_id == STATUS_REVISAO:
            self._passo_atual = PassosDeEnvio.accept_review
        if self._passo_atual == PassosDeEnvio.pre:
            card = Card(
                cv2_avs=Cv2Avs(cv2=dados['cartao_cvv']),
                pan=dados['cartao_numero'].replace(' ', ''),
                expirydate=dados['cartao_data_expiracao'],
                card_account_type=TipoDeCartao.credito
            )
            card_txn = CardTxn(card=card, method=self._passo_atual)
            txn_details = TxnDetails(
                dba=self.configuracao.usuario,
                merchantreference='{:06d}'.format(pedido.numero),
                capturemethod=MetodoDeCaptura.ecomm,
                amount=self.formatador.formata_decimal(pedido.valor_total)
            )
            if self.configuracao.usar_antifraude:
                txn_details.risk = self._anti_fraude
            else:
                txn_details.remove_atributo_da_serializacao('risk')
            if self._tem_parcelas:
                tipo = TipoDeParcelamento.sem_juros if dados['cartao_parcelas_sem_juros'] == 'true' else TipoDeParcelamento.com_juros
                instalments = Instalments(type=tipo, number=dados['cartao_parcelas'])
                txn_details.instalments = instalments
            transaction = Transaction(card_txn=card_txn, txn_details=txn_details)
            transaction.remove_atributo_da_serializacao('historic_txn')
        if self._passo_atual == PassosDeEnvio.accept_review:
            historic_txn = HistoricTxn(reference=pedido_pagamento.transacao_id, method=self._passo_atual)
            transaction = Transaction(historic_txn=historic_txn)
            transaction.remove_atributo_da_serializacao('card_txn')
        if self._passo_atual == PassosDeEnvio.fulfill:
            txn_details = TxnDetails(amount=self.formatador.formata_decimal(pedido.valor_total))
            txn_details.remove_atributo_da_serializacao('risk')
            txn_details.remove_atributo_da_serializacao('instalments')
            txn_details.remove_atributo_da_serializacao('capturemethod')
            txn_details.remove_atributo_da_serializacao('merchantreference')
            txn_details.remove_atributo_da_serializacao('dba')
            historic_txn = HistoricTxn(reference=pedido_pagamento.transacao_id, authcode=pedido_pagamento.identificador_id, method=self._passo_atual)
            transaction = Transaction(historic_txn=historic_txn, txn_details=txn_details)
            transaction.remove_atributo_da_serializacao('card_txn')

        self.authentication = Authentication(
            acquirer_code=AcquirerCode(rdcd_pv=self.configuracao.token),
            password=self.configuracao.senha
        )
        self.transaction = transaction

    @property
    def _anti_fraude(self):
        risco = Risk(
            action=Action(
                merchant_configuration=self._merchant_configuration,
                customer_details=self._customer_details
            )
        )
        return risco

    @property
    def _tem_parcelas(self):
        parcelas = self._dados.get("cartao_parcelas", 1)
        return int(parcelas) > 1

    @property
    def _merchant_configuration(self):
        return MerchantConfiguration(
            channel=ChannelValues.web,
            merchant_location="Loja Integrada",
            callback_configuration=self._callback_configuration
        )

    @property
    def _customer_details(self):
        return CustomerDetails(
            shipping_details=self._shipping_details,
            risk_details=self._risk_details,
            personal_details=self._personal_details,
            address_details=self._address_details,
            payment_details=PaymentDetails(payment_method=MeiosDePagamento.credito),
            order_details=self._order_details
        )

    @property
    def _callback_configuration(self):
        notification_url = settings.REDE_NOTIFICATION_URL.format(self.configuracao.loja_id)
        return CallbackConfiguration(
            callback_format=CallbackFormat.http,
            callback_url='{}/notificacao?referencia={}'.format(notification_url, self._pedido.numero)
        )

    @property
    def _shipping_details(self):
        return ShippingDetails(
            first_name=self.formatador.trata_unicode_com_limite(self._pedido.nome_para_entrega.split(" ")[0], limite=50),
            surname=self.formatador.trata_unicode_com_limite(self._pedido.nome_para_entrega.split(" ")[1], limite=50),
            address_line1=self.endereco(TipoEndereco.entrega),
            address_line2=self.formatador.trata_unicode_com_limite(self._pedido.endereco_entrega['complemento'], limite=60),
            city=self.formatador.trata_unicode_com_limite(self._pedido.endereco_entrega['cidade'], limite=25),
            state_province=self.formatador.trata_unicode_com_limite(self._pedido.endereco_entrega['estado'], limite=25),
            country="BR",
            zip_code=self._pedido.endereco_entrega['cep'],
            delivery_date=self.formatador.formata_data(self._pedido.provavel_data_entrega, hora=False),
            delivery_method=self._pedido.pedido_envio.envio.nome,
        )

    def endereco(self, tipo=TipoEndereco.cliente):
        if tipo == TipoEndereco.cliente:
            return "{}, {}, {}".format(
                self.formatador.trata_unicode_com_limite(self._pedido.endereco_cliente['endereco'], limite=30),
                self._pedido.endereco_cliente['numero'],
                self.formatador.trata_unicode_com_limite(self._pedido.endereco_cliente['bairro'], limite=20)
            )
        if tipo == TipoEndereco.entrega:
            return "{}, {}, {}".format(
                self.formatador.trata_unicode_com_limite(self._pedido.endereco_entrega['endereco'], limite=30),
                self._pedido.endereco_entrega['numero'],
                self.formatador.trata_unicode_com_limite(self._pedido.endereco_entrega['bairro'], limite=20)
            )
        if tipo == TipoEndereco.pagamento:
            return "{}, {}, {}".format(
                self.formatador.trata_unicode_com_limite(self._pedido.endereco_pagamento['endereco'], limite=30),
                self._pedido.endereco_pagamento['numero'],
                self.formatador.trata_unicode_com_limite(self._pedido.endereco_pagamento['bairro'], limite=20)
            )
        return ""

    @property
    def _risk_details(self):
        return RiskDetails(
            account_number=self._pedido.cliente_id,
            email_address=self.formatador.trata_email_com_mais(self._pedido.cliente['email']),
            session_id=self._dados["session_id"],
            ip_address=self._dados["ip_address"]
        )

    @property
    def _personal_details(self):
        return PersonalDetails(
            first_name=self.formatador.trata_unicode_com_limite(self._pedido.cliente_nome_ascii.split(" ")[0], limite=32),
            surname=self.formatador.trata_unicode_com_limite(self._pedido.cliente_nome_ascii.split(" ")[1], limite=32),
            telephone="{}{}".format(*self._pedido.cliente_telefone),
            date_of_birth=self.formatador.formata_data(self._pedido.cliente.data_nascimento, hora=False),
            id_number=self._pedido.cliente_documento,
        )

    @property
    def _address_details(self):
        return AddressDetails(
            address_line1=self.endereco(),
            address_line2=self.formatador.trata_unicode_com_limite(self._pedido.endereco_cliente['complemento'], limite=60),
            city=self.formatador.trata_unicode_com_limite(self._pedido.endereco_cliente['cidade'], limite=25),
            state_province=self.formatador.trata_unicode_com_limite(self._pedido.endereco_cliente['estado'], limite=25),
            country="BR",
            zip_code=self._pedido.endereco_cliente['cep']
        )

    @property
    def _order_details(self):
        return OrderDetails(
            discount_value=self.formatador.formata_decimal(self._pedido.valor_desconto, em_centavos=True),
            proposition_date=self.formatador.formata_data(self._pedido.provavel_data_envio, hora=False),
            billing_details=self._billing_details,
            line_items=LineItems(
                item=self._items
            )
        )

    @property
    def _billing_details(self):
        return BillingDetails(
            name=self._dados["cartao_nome"],
            address_line1=self.endereco(TipoEndereco.pagamento),
            address_line2=self.formatador.trata_unicode_com_limite(self._pedido.endereco_pagamento['complemento'], limite=60),
            city=self.formatador.trata_unicode_com_limite(self._pedido.endereco_pagamento['cidade'], limite=25),
            state_province=self.formatador.trata_unicode_com_limite(self._pedido.endereco_pagamento['estado'], limite=25),
            country="BR",
            zip_code=self._pedido.endereco_pagamento['cep']
        )

    @property
    def _items(self):
        return [
            Item(
                product_code=self.formatador.trata_unicode_com_limite(item.sku, limite=50),
                product_description=self.formatador.trata_unicode_com_limite(item.nome, limite=50),
                order_quantity=self.formatador.formata_decimal(item.quantidade, como_int=True),
                unit_price=self.formatador.formata_decimal(item.preco_venda, em_centavos=True)
            )
            for item in self._pedido.itens
        ]


class ConfiguracaoMeioPagamento(entidades.ConfiguracaoMeioPagamento):
    _campos = ['ativo', 'aplicacao', 'usuario', 'token', 'senha', 'usar_antifraude', 'juros_valor', 'valor_minimo_aceitado', 'valor_minimo_parcela', 'mostrar_parcelamento', 'parcelas_sem_juros', 'maximo_parcelas']
    _codigo_gateway = CODIGO_GATEWAY

    def __init__(self, loja_id, codigo_pagamento=None):
        super(ConfiguracaoMeioPagamento, self).__init__(loja_id, codigo_pagamento)
        self.preencher_gateway(self._codigo_gateway, self._campos)
        self.formulario = cadastro.FormularioRede()
